<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */
namespace Ess\M2eProUpdater\Block\Adminhtml\Configuration;

use Ess\M2eProUpdater\Block\Adminhtml\AbstractBlock;

class Changelog extends AbstractBlock
{
    protected $_moduleCode;

    protected $_template = 'Ess_M2eProUpdater::configuration/changelog.phtml';

    //########################################

    public function __construct(
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        \Magento\Backend\Block\Template\Context $context,
        $moduleCode,
        array $data = []
    )
    {
        parent::__construct($helperFactory, $modelFactory, $context);

        $this->_moduleCode = $moduleCode;
    }

    //########################################

    public function getAffectedReleases()
    {
        /** @var \Ess\M2eProUpdater\Helper\Module|\Ess\M2eProUpdater\Helper\M2ePro $helper */
        $helper = $this->_moduleCode == \Ess\M2eProUpdater\Helper\Module::IDENTIFIER
            ? $this->helperFactory->getObject('Module')
            : $this->helperFactory->getObject('M2ePro');

        $currentVersion = $helper->getCurrentVersion();
        $latestVersion  = $helper->getLatestAvailableVersion();

        $affected = array();
        foreach ($helper->getReleasesInfo() as $version => $release) {

            if (version_compare($version, $currentVersion, '<=') ||
                version_compare($version, $latestVersion, '>'))
            {
                continue;
            }

            $affected[$version] = $release;
        }

        uksort($affected, function($a, $b) {
            return version_compare($b, $a);
        });

        return $affected;
    }

    public function hasChangeLogsRecords($release)
    {
        $result = array_filter($release['changelog'], function($elements){
            return !empty($elements);
        });

        return !empty($result);
    }

    //########################################
}