<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Block\Adminhtml\Configuration\Field\AutomaticUpgrade;

use Ess\M2eProUpdater\Helper\Config;
use Ess\M2eProUpdater\Block\Adminhtml\Configuration\AbstractField;

class DeclineUpgrade extends AbstractField
{
    //########################################

    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        if (!$this->isVisible()) {
            return '';
        }

        return parent::render($element);
    }

    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $jsUrls = json_encode(
            array(
                'configuration/decline' => $this->getUrl('M2eProUpdater/configuration/save', [
                    'path'  => base64_encode(Config::PREPARE_UPGRADE_TASK_ALLOWED_PATH),
                    'value' => base64_encode('0')
                ])
            )
        );

        $button = $this->getLayout()->createBlock(\Magento\Backend\Block\Widget\Button::class);
        $button->setData(
            [
                'id'      => 'decline_upgrade_button',
                'label'   => $this->helperFactory->getObject('M2ePro')->isInstalled()
                    ? __('Cancel Upgrade')
                    : __('Cancel Installation'),
                'class'   => 'primary',
                'onclick' => "require([
                    'M2eProUpdater/Configuration'
                ],function (ConfigurationObj) {
                    
                    ConfigurationObj.decline();
                });",
            ]
        );

        return <<<HTML
<script type="text/javascript">
require([
    'M2eProUpdater/Configuration'
],function (ConfigurationObj) {
    Object.extend(ConfigurationObj.urls, {$jsUrls});
});
</script>
HTML
            . $button->toHtml();
    }

    //########################################

    private function isVisible()
    {
        /** @var \Ess\M2eProUpdater\Helper\Config $helper */
        $helper = $this->helperFactory->getObject('Config');
        return $helper->isPrepareUpgradeTaskAllowed();
    }

    //########################################
}