<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Block\Adminhtml\Configuration\Field\AutomaticUpgrade;

use Ess\M2eProUpdater\Helper\Config;
use Ess\M2eProUpdater\Block\Adminhtml\Configuration\AbstractField;

class ScheduleUpgrade extends AbstractField
{
    //########################################

    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        if (!$this->isVisible()) {
            return '';
        }

        return parent::render($element);
    }

    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $isInstalled = $this->helperFactory->getObject('M2ePro')->isInstalled();
        $popupTitle = $isInstalled ? __('Confirm M2E Pro Upgrade') : __('Confirm M2E Pro Installation');
        $buttonTitle = $isInstalled ? __('Schedule Upgrade') : __('Schedule Installation');
        $confirmButtonTitle = $isInstalled ? __('Upgrade') : __('Install');

        /** @var \Magento\Backend\Block\Widget\Button $button */
        $button = $this->getLayout()->createBlock(\Magento\Backend\Block\Widget\Button::class);
        $button->setData(
            array(
                'id'      => 'schedule_upgrade_button',
                'label'   => $buttonTitle,
                'class'   => 'primary',
                'onclick' => "require([
    'M2eProUpdater/Configuration'
],function (ConfigurationObj) {
    
    ConfigurationObj.showSchedulePopup('{$popupTitle}', '{$confirmButtonTitle}')
});",
            )
        );

        return $this->getPopupHtml() . $button->toHtml();
    }

    private function getPopupHtml()
    {
        $url = $this->helperFactory->getObject('Support')->getDocumentationUrl(null, 'x/bQM0AQ');
        $isInstalled = $this->helperFactory->getObject('M2ePro')->isInstalled();
        $url .= $isInstalled
            ? '#viaM2EProInstallation/UpgradeModule-M2EProUpgrade'
            : '#viaM2EProInstallation/UpgradeModule-M2EProInstallation';

        $action = $isInstalled ? 'upgrade' : 'installation';

        $jsUrls = json_encode(
            array(
                'configuration/schedule' => $this->getUrl(
                    'M2eProUpdater/configuration/save',
                    array(
                        'path'  => base64_encode(Config::PREPARE_UPGRADE_TASK_ALLOWED_PATH),
                        'value' => base64_encode('1')
                    )
                ),
            )
        );

        return <<<HTML
<script type="text/javascript">
require([
    'M2eProUpdater/Configuration'
],function (ConfigurationObj) {
    Object.extend(ConfigurationObj.urls, {$jsUrls});
});
</script>

<div id="m2epro_updater_schedule_upgrade_popup_content" style="display: none;">

    <p style="padding: 10px;">
        It can take up to 15 minutes to prepare the system for an update.
        Your Magento instance will stay in maintenance mode until the Extension {$action} is finished.<br><br>

        Please ensure that all the <a href="{$url}" target="_blank">preliminary requirements</a>
        for the {$action} are met before you confirm the action.
    </p>

</div>
HTML;
    }

    //########################################

    private function isVisible()
    {
        /** @var \Ess\M2eProUpdater\Helper\Config $configHelper */
        $configHelper = $this->helperFactory->getObject('Config');
        /** @var \Ess\M2eProUpdater\Helper\M2ePro $m2eProHelper */
        $m2eProHelper = $this->helperFactory->getObject('M2ePro');

        if ($m2eProHelper->isInstalledViaComposer()) {
            return false;
        }

        if ($configHelper->isPrepareUpgradeTaskAllowed() || $configHelper->isDoUpgradeTaskAllowed()) {
            return false;
        }

        $currentVersion = $m2eProHelper->getCurrentVersion();
        $latestVersion  = $m2eProHelper->getLatestAvailableVersion();

        if ($latestVersion && version_compare($latestVersion, $currentVersion, '>')) {
            return true;
        }

        return false;
    }

    //########################################
}