<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Block\Adminhtml\Configuration\Field\Information;

use \Ess\M2eProUpdater\Block\Adminhtml\Configuration\AbstractField;

class M2eProLatestVersion extends AbstractField
{
    //########################################

    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $urls = json_encode(array(
            'configuration/getChangeLogHtml' => $this->getUrl(
                'M2eProUpdater/configuration/getChangeLogHtml'
            ),
        ));

        $javascript = <<<HTML
<script type="text/javascript">

require([
    'M2eProUpdater/Configuration'
],function (ConfigurationObj) {
    Object.extend(ConfigurationObj.urls, {$urls});
});

</script>
HTML;
        return $javascript . parent::render($element);
    }

    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        /** @var \Ess\M2eProUpdater\Helper\M2ePro $helper */
        $helper = $this->helperFactory->getObject('M2ePro');

        $currentVersion = $helper->getCurrentVersion();
        $latestVersion  = $helper->getLatestAvailableVersion();

        if ($latestVersion && (version_compare($latestVersion, $currentVersion, '>'))) {

            $latestVersion .= '  ' . __('[New]');
            $changelogWorld = __('[changelog]');
            $moduleCode = \Ess\M2eProUpdater\Helper\M2ePro::IDENTIFIER;

            $afterHtml = <<<HTML
<a href="javascript:void(0)" 
   onclick="require([
    'M2eProUpdater/Configuration'
],function (ConfigurationObj) {
    ConfigurationObj.showModuleChangelogPopup('{$moduleCode}');
});">{$changelogWorld}</a>
HTML;

            $element->setBold(true);
            $element->setData('after_element_html', $afterHtml);

        } else if (!$latestVersion) {
            $latestVersion = __('N/A');
        }

        $element->setValue($latestVersion);
        return parent::_getElementHtml($element);
    }

    //########################################
}