<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Model;

class Downloader extends \Ess\M2eProUpdater\Model\AbstractModel
{
    const VISIBILITY_PUBLIC = 'public';
    const VISIBILITY_BETA   = 'beta_testers';

    const BUILD_FORMAT_ZIP = 'zip';

    const CHANGELOG_FORMAT_OLD = 1;
    const CHANGELOG_FORMAT_NEW = 2;

    private $extensionCode;
    private $visibility;

    //########################################

    public function __construct(
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        $extensionCode = NULL,
        $visibility = NULL,
        array $data = []
    ){
        parent::__construct($helperFactory, $modelFactory, $data);

        $this->extensionCode = $extensionCode;
        $this->visibility    = $visibility;

        $this->validate();
    }

    private function validate()
    {
        if (!isset($this->extensionCode) ||
            !in_array($this->extensionCode, array(\Ess\M2eProUpdater\Helper\Module::IDENTIFIER,
                                                  \Ess\M2eProUpdater\Helper\M2ePro::IDENTIFIER))) {

            throw new \Exception(sprintf(
                'Unsupported extension code [%s]', $this->extensionCode
            ));
        }
    }

    //########################################

    /**
     * @param mixed $version (NULL - Latest Version; * All releases; string - Exact Version)
     * @return array
     */
    public function getReleaseInfo($version = NULL)
    {
        $url = $this->getServerUrl('get-release-metadata.php', array(
            'version'          => $version,
            'visibility'       => $this->visibility,
            'changelog_format' => self::CHANGELOG_FORMAT_OLD
        ));

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_URL, $url);

        $data = curl_exec($ch);
        curl_close($ch);

        return (array)json_decode($data, true);
    }

    /**
     * @param mixed $version (NULL - Latest Version; string - Exact Version)
     * @param string $format (zip, connect1, connect2)
     * @return string
     */
    public function getPackageUrl($version = NULL, $format = self::BUILD_FORMAT_ZIP)
    {
        return $this->getServerUrl('get-release-version.php', array(
            'version'    => $version,
            'visibility' => $this->visibility,
            'format'     => $format
        ));
    }

    //########################################

    private function getServerUrl($relativeUrl, array $query = array())
    {
        $serverLocations = array(
            \Ess\M2eProUpdater\Helper\Module::IDENTIFIER => 'https://download.m2epro.com/extension_updater/magento_2/',
            \Ess\M2eProUpdater\Helper\M2ePro::IDENTIFIER => 'https://download.m2epro.com/extension/magento_2/',
        );

        return $serverLocations[$this->extensionCode] . $relativeUrl .'?'. http_build_query($query);
    }

    //########################################
}