<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Model\System;

use \Magento\Framework\Notification\MessageInterface;
use \Ess\M2eProUpdater\Plugin\Config\Magento\Config\Model\Config\Structure\Data as Structure;

class Message extends \Ess\M2eProUpdater\Model\AbstractModel implements MessageInterface
{
    //########################################

    /** @var \Magento\Framework\UrlInterface */
    protected $urlBuilder;

    /** @var \Magento\Backend\Model\Auth */
    protected $authorization;

    //########################################

    public function __construct(
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Backend\Model\Auth $authorization,
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory
    ) {
        parent::__construct($helperFactory, $modelFactory);
        $this->urlBuilder    = $urlBuilder;
        $this->authorization = $authorization;
    }

    //########################################

    public function getIdentity()
    {
        return md5('m2epro-latest-version-' . $this->helperFactory->getObject('M2ePro')->getLatestAvailableVersion());
    }

    public function isDisplayed()
    {
        if (!$this->authorization->isLoggedIn()) {
            return false;
        }

        /** @var \Ess\M2eProUpdater\Helper\Config $helper */
        $helper = $this->helperFactory->getObject('Config');
        if (!$helper->isNotificationMagentoSystemNotification()) {
            return false;
        }

        /** @var \Ess\M2eProUpdater\Helper\M2ePro $helper */
        $helper = $this->helperFactory->getObject('M2ePro');
        if (!$helper->getCurrentVersion() ||
            version_compare($helper->getCurrentVersion(), $helper->getLatestAvailableVersion(), '>=')
        ) {
            return false;
        }

        return true;
    }

    public function getText()
    {
        $link = $this->urlBuilder->getUrl('adminhtml/system_config/edit', [
            'section' => Structure::INSTALLATION_UPGRADE_SECTION
        ]);

        return <<<HTML
<div class="notification-global">
    A new version of M2E Pro is available. Navigate <i>Stores > Configuration > Multi Channels > Upgrade</i> to
    schedule an upgrade.
    Click <a target="_blank" href="{$link}">here</a> to get to the <i>Upgrade</i> page now.
</div>
HTML;
    }

    public function getSeverity()
    {
        return \Magento\Framework\Notification\MessageInterface::SEVERITY_CRITICAL;
    }

    //########################################
}