<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Model;

use Magento\Framework\App\Filesystem\DirectoryList;

class Updater extends \Ess\M2eProUpdater\Model\AbstractModel
{
    /** @var \Magento\Framework\App\Filesystem\DirectoryList $directoryList */
    private $directoryList;

    /** @var \Magento\Framework\Filesystem\Directory\WriteFactory */
    private $directoryWriterFactory;

    //########################################

    public function __construct(
        \Magento\Framework\HTTP\Client\Curl $curlClient,
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
        \Magento\Framework\Filesystem\Directory\WriteFactory $directoryWriterFactory,
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        $data = []
    ) {
        parent::__construct($helperFactory, $modelFactory, $data);

        $this->directoryList = $directoryList;
        $this->directoryWriterFactory = $directoryWriterFactory;
    }

    //########################################

    public function validate()
    {
        $path = $this->directoryList->getPath(DirectoryList::APP) . '/code/Ess/';
        $directory = $this->directoryWriterFactory->create($path);

        $directory->writeFile('permissions_check.txt', '1');
        $directory->delete('permissions_check.txt');
    }

    public function prepareNewPackage()
    {
        try {
            !is_file($this->getTemporaryFilePath()) && $this->_download();
            $this->_unpack();
        } catch (\Exception $e) {
            is_file($this->getTemporaryFilePath()) && unlink($this->getTemporaryFilePath());
            throw $e;
        }
    }

    public function updatePackage()
    {
        $this->_clearTargetDirectory();
        $this->_copy();
        $this->_removeUnpacked();
    }

    //########################################

    protected function _clearTargetDirectory()
    {
        /** @var \Ess\M2eProUpdater\Helper\M2ePro $m2eProHelper */
        $m2eProHelper = $this->helperFactory->getObject('M2ePro');
        $directory = $this->directoryWriterFactory->create($m2eProHelper->getCodeDirectoryPath());

        $directory->delete();
    }

    protected function _removeUnpacked()
    {
        $directory = $this->directoryWriterFactory->create($this->getUnpackedDirectoryPath());
        $directory->delete();

        $directory = $this->directoryWriterFactory->create(
            $this->helperFactory->getObject('Module')->getTemporaryDirectoryPath()
        );

        $directory->delete($this->getTemporaryFileName());
    }

    //########################################

    protected function _download()
    {
        /** @var \Ess\M2eProUpdater\Model\Downloader $downloader */
        $downloader = $this->modelFactory->getObject('Downloader', [
            'visibility'    => $this->helperFactory->getObject('M2ePro')->getPackageVisibility(),
            'extensionCode' => \Ess\M2eProUpdater\Helper\M2ePro::IDENTIFIER
        ]);

        $targetFileHandler = fopen($this->getTemporaryFilePath(), 'wb');

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_URL, $downloader->getPackageUrl());
        curl_setopt($ch, CURLOPT_FILE, $targetFileHandler);

        curl_exec($ch);
        curl_close($ch);

        fclose($targetFileHandler);
    }

    protected function _unpack()
    {
        /** @var \Ess\M2eProUpdater\Helper\Module $moduleHelper */
        $moduleHelper = $this->helperFactory->getObject('Module');
        $directory = $this->directoryWriterFactory->create($moduleHelper->getTemporaryDirectoryPath());

        if (!$directory->isFile($this->getTemporaryFileName())) {
            throw new \Exception(sprintf(
                'Unable to locate the source code ZIP [%s].', $this->getTemporaryFilePath()
            ));
        }

        $zipObject = new \ZipArchive();
        $zipObject->open($this->getTemporaryFilePath());

        $zipObject->extractTo($this->getUnpackedDirectoryPath());
        $zipObject->close();
    }

    protected function _copy()
    {
        $targetDirectory = $this->directoryWriterFactory->create($this->directoryList->getRoot());
        $sourceDirectory = $this->directoryWriterFactory->create($this->getUnpackedDirectoryPath());

        foreach ($sourceDirectory->readRecursively() as $path) {

            if (!$sourceDirectory->getDriver()->isFile($sourceDirectory->getAbsolutePath($path))) {
                continue;
            }

            $sourceDirectory->copyFile($path, $path, $targetDirectory);
        }

        $sourceDirectory->delete();
    }

    //########################################

    protected function getUnpackedDirectoryPath()
    {
        $path = $this->helperFactory->getObject('Module')->getTemporaryDirectoryPath() . '/unpacked/';
        $directory = $this->directoryWriterFactory->create($path);

        if (!$directory->isExist()) {
            $directory->create();
        }

        return $path;
    }

    protected function getTemporaryFilePath()
    {
        return $this->helperFactory->getObject('Module')->getTemporaryDirectoryPath() .'/'.
               $this->getTemporaryFileName();
    }

    protected function getTemporaryFileName()
    {
        $latestRelease = $this->helperFactory->getObject('M2ePro')->getLatestAvailableReleaseInfo();
        return isset($latestRelease['files']['zip']) ? $latestRelease['files']['zip'] : 'new-version.zip';
    }

    //########################################
}